@@ -4,8 +4,6 @@
 
 ''' "Metrique User Management" related funtions '''
 
-import logging
-logger = logging.getLogger(__name__)
 import os
 
 from metriqueu.utils import set_default
@@ -44,7 +42,7 @@ def register(self, username=None, password=None, logon_as=True):
         return result
 
 
-def remove(self, username=None):
+def remove(self, username=None, quiet=False):
     '''
     Register new user
 
@@ -52,7 +50,13 @@ def remove(self, username=None):
     '''
     username = set_default(username, self.config.username)
     cmd = os.path.join(username, 'remove')
-    result = self._delete(cmd, username=username, api_url=False)
+    try:
+        result = self._delete(cmd, username=username, api_url=False)
+    except Exception:
+        if quiet:
+            result = None
+        else:
+            raise
     return result
 
 
@@ -126,12 +130,11 @@ def update_passwd(self, new_password, old_password=None,
 
 
 # FIXME: prefix api_url with _ to indicate it's a special kwarg
-def update_profile(self, username=None, backup=False, email=None):
-    username = set_default(username, self.config.username)
+def update_profile(self, username=None, gnupg=None):
+    username = username or self.config.username
     cmd = os.path.join(username, 'update_profile')
     result = self._post(cmd,
-                        backup=backup,
-                        email=email,
+                        gnupg=gnupg,
                         api_url=False)
     return result
 
